import { TypedEmitter } from "tiny-typed-emitter";
import { Logger } from "ts-log";
import { EufySecurityEvents, EufySecurityConfig } from "./interfaces";
import { HTTPApi } from "./http/api";
import { Station } from "./http/station";
import { PushNotificationService } from "./push/service";
import { Credentials } from "./push/models";
import { Device } from "./http/device";
import { P2PConnectionType } from "./p2p/types";
export declare class EufySecurity extends TypedEmitter<EufySecurityEvents> {
    private config;
    private log;
    private api;
    private stations;
    private devices;
    private readonly P2P_REFRESH_INTERVAL_MIN;
    private cameraMaxLivestreamSeconds;
    private cameraStationLivestreamTimeout;
    private cameraCloudLivestreamTimeout;
    private pushService;
    private pushCloudRegistered;
    private pushCloudChecked;
    private persistentFile;
    private persistentData;
    private connected;
    private refreshEufySecurityCloudTimeout?;
    private refreshEufySecurityP2PTimeout;
    constructor(config: EufySecurityConfig, log?: Logger);
    getPushService(): PushNotificationService;
    private addStation;
    private removeStation;
    private updateStation;
    private addDevice;
    private removeDevice;
    private updateDevice;
    getDevices(): Array<Device>;
    getDevice(deviceSN: string): Device;
    getStationDevice(stationSN: string, channel: number): Device;
    getStations(): Array<Station>;
    getStation(stationSN: string): Station;
    getApi(): HTTPApi;
    connectToStation(stationSN: string, p2pConnectionType?: P2PConnectionType): Promise<void>;
    isStationConnected(stationSN: string): boolean;
    isStationEnergySavingDevice(stationSN: string): boolean;
    private handleHubs;
    private onStationConnect;
    private onStationClose;
    private handleDevices;
    refreshCloudData(): Promise<void>;
    close(): void;
    setCameraMaxLivestreamDuration(seconds: number): void;
    getCameraMaxLivestreamDuration(): number;
    registerPushNotifications(credentials?: Credentials, persistentIds?: string[]): Promise<void>;
    connect(verifyCodeOrCaptcha?: string | null, captchaId?: string | null): Promise<boolean>;
    getPushPersistentIds(): string[];
    private updateDeviceProperties;
    private onAPIClose;
    private onAPIConnect;
    startStationLivestream(deviceSN: string): Promise<void>;
    startCloudLivestream(deviceSN: string): Promise<void>;
    stopStationLivestream(deviceSN: string): Promise<void>;
    stopCloudLivestream(deviceSN: string): Promise<void>;
    private writePersistentData;
    private saveAPIBase;
    private saveCloudToken;
    private savePushCredentials;
    private savePushPersistentIds;
    getVersion(): string;
    isPushConnected(): boolean;
    isConnected(): boolean;
    private processInvitations;
    private onPushMessage;
    startStationDownload(deviceSN: string, path: string, cipherID: number): Promise<void>;
    cancelStationDownload(deviceSN: string): Promise<void>;
    getConfig(): EufySecurityConfig;
    setDeviceProperty(deviceSN: string, name: string, value: unknown): Promise<void>;
    setStationProperty(stationSN: string, name: string, value: unknown): Promise<void>;
    private onStartStationLivestream;
    private onStopStationLivestream;
    private onStartStationRTSPLivestream;
    private onStopStationRTSPLivestream;
    private onStationStartDownload;
    private onStationFinishDownload;
    private onStationCommandResult;
    private onStationRtspUrl;
    private onStationGuardMode;
    private onStationCurrentMode;
    private onStationPropertyChanged;
    private onStationPropertyRenewed;
    private onStationRawPropertyChanged;
    private onStationRawPropertyRenewed;
    private onStationAlarmEvent;
    private onDevicePropertyChanged;
    private onDevicePropertyRenewed;
    private onDeviceRawPropertyChanged;
    private onDeviceRawPropertyRenewed;
    private onDeviceCryingDetected;
    private onDeviceSoundDetected;
    private onDevicePetDetected;
    private onDeviceMotionDetected;
    private onDevicePersonDetected;
    private onDeviceRings;
    private onDeviceLocked;
    private onDeviceOpen;
    private onDeviceReady;
    private onStationRuntimeState;
    private onStationChargingState;
    private onStationWifiRssi;
    private onCaptchaRequest;
    private onFloodlightManualSwitch;
}
