import { Method } from "got";
import { TypedEmitter } from "tiny-typed-emitter";
import { Logger } from "ts-log";
import { TrustDevice, Cipher, EventRecordResponse, ConfirmInvite, SensorHistoryEntry, ApiResponse } from "./models";
import { HTTPApiEvents, Ciphers, FullDevices, Hubs, Voices, Invites } from "./interfaces";
import { AuthResult, EventFilterType, PublicKeyType, VerfyCodeTypes } from "./types";
export declare class HTTPApi extends TypedEmitter<HTTPApiEvents> {
    private apiBase;
    private username;
    private password;
    private userId;
    private ecdh;
    private serverPublicKey;
    private token;
    private tokenExpiration;
    private trustedTokenExpiration;
    private log;
    private connected;
    private devices;
    private hubs;
    private headers;
    constructor(username: string, password: string, log?: Logger);
    private invalidateToken;
    setPhoneModel(model: string): void;
    getPhoneModel(): string;
    setCountry(country: string): void;
    getCountry(): string;
    setLanguage(language: string): void;
    getLanguage(): string;
    authenticate(verifyCodeOrCaptcha?: string | null, captchaId?: string | null): Promise<AuthResult>;
    sendVerifyCode(type?: VerfyCodeTypes): Promise<boolean>;
    listTrustDevice(): Promise<Array<TrustDevice>>;
    addTrustDevice(verifyCode: string): Promise<boolean>;
    updateDeviceInfo(): Promise<void>;
    request(method: Method, endpoint: string, data?: any, headers?: Record<string, string>): Promise<ApiResponse>;
    checkPushToken(): Promise<boolean>;
    registerPushToken(token: string): Promise<boolean>;
    setParameters(stationSN: string, deviceSN: string, params: {
        paramType: number;
        paramValue: any;
    }[]): Promise<boolean>;
    getCiphers(cipherIDs: Array<number>, userID: string): Promise<Ciphers>;
    getVoices(deviceSN: string): Promise<Voices>;
    getCipher(cipherID: number, userID: string): Promise<Cipher>;
    getLog(): Logger;
    getDevices(): FullDevices;
    getHubs(): Hubs;
    getToken(): string | null;
    getTokenExpiration(): Date | null;
    getTrustedTokenExpiration(): Date;
    setToken(token: string): void;
    setTokenExpiration(tokenExpiration: Date): void;
    getAPIBase(): string;
    setAPIBase(apiBase: string): void;
    setOpenUDID(openudid: string): void;
    setSerialNumber(serialnumber: string): void;
    private _getEvents;
    getVideoEvents(startTime: Date, endTime: Date, filter?: EventFilterType, maxResults?: number): Promise<Array<EventRecordResponse>>;
    getAlarmEvents(startTime: Date, endTime: Date, filter?: EventFilterType, maxResults?: number): Promise<Array<EventRecordResponse>>;
    getHistoryEvents(startTime: Date, endTime: Date, filter?: EventFilterType, maxResults?: number): Promise<Array<EventRecordResponse>>;
    getAllVideoEvents(filter?: EventFilterType, maxResults?: number): Promise<Array<EventRecordResponse>>;
    getAllAlarmEvents(filter?: EventFilterType, maxResults?: number): Promise<Array<EventRecordResponse>>;
    getAllHistoryEvents(filter?: EventFilterType, maxResults?: number): Promise<Array<EventRecordResponse>>;
    isConnected(): boolean;
    getInvites(): Promise<Invites>;
    confirmInvites(confirmInvites: Array<ConfirmInvite>): Promise<boolean>;
    getPublicKey(deviceSN: string, type: PublicKeyType): Promise<string>;
    getSensorHistory(stationSN: string, deviceSN: string): Promise<Array<SensorHistoryEntry>>;
}
