/// <reference types="node" />
import { TypedEmitter } from "tiny-typed-emitter";
import { Logger } from "ts-log";
import { HTTPApi } from "./api";
import { CommandName, DeviceEvent } from "./types";
import { FullDeviceResponse } from "./models";
import { DeviceEvents, PropertyValue, PropertyValues, PropertyMetadataAny, IndexedProperty, RawValues, RawValue } from "./interfaces";
import { PushMessage } from "../push";
export declare abstract class Device extends TypedEmitter<DeviceEvents> {
    protected api: HTTPApi;
    protected rawDevice: FullDeviceResponse;
    protected log: Logger;
    protected eventTimeouts: Map<DeviceEvent, NodeJS.Timeout>;
    protected properties: PropertyValues;
    private rawProperties;
    private ready;
    constructor(api: HTTPApi, device: FullDeviceResponse);
    getRawDevice(): FullDeviceResponse;
    update(device: FullDeviceResponse): void;
    protected updateProperty(name: string, value: PropertyValue): boolean;
    updateRawProperties(values: RawValues): void;
    protected processCustomParameterChanged(metadata: PropertyMetadataAny, oldValue: PropertyValue, newValue: PropertyValue): void;
    updateRawProperty(type: number, value: RawValue): boolean;
    protected convertRawPropertyValue(property: PropertyMetadataAny, value: RawValue): PropertyValue;
    getPropertyMetadata(name: string): PropertyMetadataAny;
    getPropertyValue(name: string): PropertyValue;
    getRawProperty(type: number): RawValue;
    getRawProperties(): RawValues;
    getProperties(): PropertyValues;
    getPropertiesMetadata(): IndexedProperty;
    hasProperty(name: string): boolean;
    getCommands(): Array<CommandName>;
    hasCommand(name: CommandName): boolean;
    processPushNotification(_message: PushMessage, _eventDurationSeconds: number): void;
    setCustomPropertyValue(name: string, value: PropertyValue): void;
    destroy(): void;
    protected clearEventTimeout(eventType: DeviceEvent): void;
    static isCamera(type: number): boolean;
    static hasBattery(type: number): boolean;
    static isStation(type: number): boolean;
    static isSensor(type: number): boolean;
    static isKeyPad(type: number): boolean;
    static isDoorbell(type: number): boolean;
    static isWiredDoorbell(type: number): boolean;
    static isIndoorCamera(type: number): boolean;
    static isFloodLight(type: number): boolean;
    static isLock(type: number): boolean;
    static isLockBasic(type: number): boolean;
    static isLockBasicNoFinger(type: number): boolean;
    static isLockAdvanced(type: number): boolean;
    static isLockAdvancedNoFinger(type: number): boolean;
    static isBatteryDoorbell(type: number): boolean;
    static isBatteryDoorbell2(type: number): boolean;
    static isSoloCamera(type: number): boolean;
    static isSoloCameraPro(type: number): boolean;
    static isSoloCameraSpotlight1080(type: number): boolean;
    static isSoloCameraSpotlight2k(type: number): boolean;
    static isSoloCameraSpotlightSolar(type: number): boolean;
    static isSoloCameras(type: number): boolean;
    static isIndoorOutdoorCamera1080p(type: number): boolean;
    static isIndoorOutdoorCamera1080pNoLight(type: number): boolean;
    static isIndoorOutdoorCamera2k(type: number): boolean;
    static isCamera2(type: number): boolean;
    static isCamera2C(type: number): boolean;
    static isCamera2Pro(type: number): boolean;
    static isCamera2CPro(type: number): boolean;
    static isCamera2Product(type: number): boolean;
    static isEntrySensor(type: number): boolean;
    static isMotionSensor(type: number): boolean;
    static isIntegratedDeviceBySn(sn: string): boolean;
    static isSoloCameraBySn(sn: string): boolean;
    isCamera(): boolean;
    isFloodLight(): boolean;
    isDoorbell(): boolean;
    isWiredDoorbell(): boolean;
    isLock(): boolean;
    isLockBasic(): boolean;
    isLockBasicNoFinger(): boolean;
    isLockAdvanced(): boolean;
    isLockAdvancedNoFinger(): boolean;
    isBatteryDoorbell(): boolean;
    isBatteryDoorbell2(): boolean;
    isSoloCamera(): boolean;
    isSoloCameraPro(): boolean;
    isSoloCameraSpotlight1080(): boolean;
    isSoloCameraSpotlight2k(): boolean;
    isSoloCameraSpotlightSolar(): boolean;
    isIndoorOutdoorCamera1080p(): boolean;
    isIndoorOutdoorCamera1080pNoLight(): boolean;
    isIndoorOutdoorCamera2k(): boolean;
    isSoloCameras(): boolean;
    isCamera2(): boolean;
    isCamera2C(): boolean;
    isCamera2Pro(): boolean;
    isCamera2CPro(): boolean;
    isCamera2Product(): boolean;
    isEntrySensor(): boolean;
    isKeyPad(): boolean;
    isMotionSensor(): boolean;
    isIndoorCamera(): boolean;
    hasBattery(): boolean;
    getDeviceKey(): string;
    getDeviceType(): number;
    getHardwareVersion(): string;
    getSoftwareVersion(): string;
    getModel(): string;
    getName(): string;
    getSerial(): string;
    getStationSerial(): string;
    setParameters(params: {
        paramType: number;
        paramValue: any;
    }[]): Promise<boolean>;
    getChannel(): number;
    getStateID(state: string, level?: number): string;
    abstract getStateChannel(): string;
    getWifiRssi(): PropertyValue;
    getStoragePath(filename: string): string;
    isEnabled(): PropertyValue;
}
export declare class Camera extends Device {
    private _isStreaming;
    constructor(api: HTTPApi, device: FullDeviceResponse);
    getStateChannel(): string;
    protected convertRawPropertyValue(property: PropertyMetadataAny, value: RawValue): PropertyValue;
    getLastCameraImageURL(): PropertyValue;
    getMACAddress(): string;
    startDetection(): Promise<void>;
    startStream(): Promise<string>;
    stopDetection(): Promise<void>;
    stopStream(): Promise<void>;
    getState(): PropertyValue;
    isStreaming(): boolean;
    close(): Promise<void>;
    getLastChargingDays(): number;
    getLastChargingFalseEvents(): number;
    getLastChargingRecordedEvents(): number;
    getLastChargingTotalEvents(): number;
    getBatteryValue(): PropertyValue;
    getBatteryTemperature(): PropertyValue;
    isMotionDetectionEnabled(): PropertyValue;
    isLedEnabled(): PropertyValue;
    isAutoNightVisionEnabled(): PropertyValue;
    isRTSPStreamEnabled(): PropertyValue;
    isAntiTheftDetectionEnabled(): PropertyValue;
    getWatermark(): PropertyValue;
    isMotionDetected(): boolean;
    isPersonDetected(): boolean;
    getDetectedPerson(): string;
    processPushNotification(message: PushMessage, eventDurationSeconds: number): void;
}
export declare class SoloCamera extends Camera {
    isLedEnabled(): PropertyValue;
    isMotionDetectionEnabled(): PropertyValue;
    processPushNotification(message: PushMessage, eventDurationSeconds: number): void;
}
export declare class IndoorCamera extends Camera {
    constructor(api: HTTPApi, device: FullDeviceResponse);
    isLedEnabled(): PropertyValue;
    isMotionDetectionEnabled(): PropertyValue;
    isPetDetectionEnabled(): PropertyValue;
    isSoundDetectionEnabled(): PropertyValue;
    isPetDetected(): boolean;
    isSoundDetected(): boolean;
    isCryingDetected(): boolean;
    processPushNotification(message: PushMessage, eventDurationSeconds: number): void;
    destroy(): void;
}
export declare class DoorbellCamera extends Camera {
    constructor(api: HTTPApi, device: FullDeviceResponse);
    isRinging(): boolean;
    processPushNotification(message: PushMessage, eventDurationSeconds: number): void;
}
export declare class WiredDoorbellCamera extends DoorbellCamera {
    isLedEnabled(): PropertyValue;
    isAutoNightVisionEnabled(): PropertyValue;
    isMotionDetectionEnabled(): PropertyValue;
}
export declare class BatteryDoorbellCamera extends DoorbellCamera {
    isLedEnabled(): PropertyValue;
}
export declare class FloodlightCamera extends Camera {
    isLedEnabled(): PropertyValue;
    isMotionDetectionEnabled(): PropertyValue;
    protected convertRawPropertyValue(property: PropertyMetadataAny, value: RawValue): PropertyValue;
}
export declare class Sensor extends Device {
    getStateChannel(): string;
    getState(): PropertyValue;
}
export declare class EntrySensor extends Sensor {
    isSensorOpen(): PropertyValue;
    getSensorChangeTime(): PropertyValue;
    isBatteryLow(): PropertyValue;
    processPushNotification(message: PushMessage, eventDurationSeconds: number): void;
}
export declare class MotionSensor extends Sensor {
    static readonly MOTION_COOLDOWN_MS = 120000;
    constructor(api: HTTPApi, device: FullDeviceResponse);
    isMotionDetected(): boolean;
    getMotionSensorPIREvent(): PropertyValue;
    isBatteryLow(): PropertyValue;
    processPushNotification(message: PushMessage, eventDurationSeconds: number): void;
}
export declare class Lock extends Device {
    getStateChannel(): string;
    protected processCustomParameterChanged(metadata: PropertyMetadataAny, oldValue: PropertyValue, newValue: PropertyValue): void;
    getState(): PropertyValue;
    getBatteryValue(): PropertyValue;
    getWifiRssi(): PropertyValue;
    isLocked(): PropertyValue;
    getLockStatus(): PropertyValue;
    static encodeESLCmdOnOff(short_user_id: number, nickname: string, lock: boolean): Buffer;
    static encodeESLCmdQueryStatus(admin_user_id: string): Buffer;
    protected convertRawPropertyValue(property: PropertyMetadataAny, value: RawValue): PropertyValue;
    processPushNotification(message: PushMessage, eventDurationSeconds: number): void;
}
export declare class Keypad extends Device {
    getStateChannel(): string;
    getState(): PropertyValue;
    isBatteryLow(): PropertyValue;
    isBatteryCharging(): PropertyValue;
    protected convertRawPropertyValue(property: PropertyMetadataAny, value: RawValue): PropertyValue;
}
export declare class UnknownDevice extends Device {
    getStateChannel(): string;
}
