import { TypedEmitter } from "tiny-typed-emitter";
import { HTTPApi } from "./api";
import { AlarmTone, NotificationSwitchMode, DeviceType, FloodlightMotionTriggeredDistance, GuardMode, NotificationType, PowerSource, TimeFormat, CommandName } from "./types";
import { HubResponse } from "./models";
import { IndexedProperty, PropertyMetadataAny, PropertyValue, PropertyValues, RawValue, RawValues, StationEvents } from "./interfaces";
import { P2PConnectionType, PanTiltDirection, VideoCodec, WatermarkSetting1, WatermarkSetting2, WatermarkSetting3, WatermarkSetting4 } from "../p2p/types";
import { Device } from "./device";
import { PushMessage } from "../push/models";
export declare class Station extends TypedEmitter<StationEvents> {
    private api;
    private rawStation;
    private log;
    private p2pSession;
    private properties;
    private rawProperties;
    private ready;
    private currentDelay;
    private reconnectTimeout?;
    private terminating;
    private p2pConnectionType;
    static readonly CHANNEL: number;
    static readonly CHANNEL_INDOOR: number;
    constructor(api: HTTPApi, station: HubResponse);
    getStateID(state: string, level?: number): string;
    getStateChannel(): string;
    getRawStation(): HubResponse;
    update(station: HubResponse): void;
    private updateProperty;
    updateRawProperties(values: RawValues): void;
    updateRawProperty(type: number, value: RawValue): boolean;
    protected convertRawPropertyValue(property: PropertyMetadataAny, value: RawValue): PropertyValue;
    getPropertyMetadata(name: string): PropertyMetadataAny;
    getPropertyValue(name: string): PropertyValue;
    getRawProperty(type: number): RawValue;
    getRawProperties(): RawValues;
    getProperties(): PropertyValues;
    getPropertiesMetadata(): IndexedProperty;
    hasProperty(name: string): boolean;
    getCommands(): Array<CommandName>;
    hasCommand(name: CommandName): boolean;
    isStation(): boolean;
    isDeviceStation(): boolean;
    getDeviceType(): number;
    getHardwareVersion(): string;
    getMACAddress(): string;
    getModel(): string;
    getName(): string;
    getSerial(): string;
    getSoftwareVersion(): string;
    getIPAddress(): string;
    getLANIPAddress(): PropertyValue;
    getGuardMode(): PropertyValue;
    getCurrentMode(): PropertyValue;
    processPushNotification(message: PushMessage): void;
    isConnected(): boolean;
    close(): void;
    isEnergySavingDevice(): boolean;
    connect(): Promise<void>;
    private onFinishDownload;
    private onStartDownload;
    private onStopLivestream;
    private onStartLivestream;
    private onStopRTSPLivestream;
    private onStartRTSPLivestream;
    private onWifiRssiChanged;
    private onRTSPUrl;
    private onESLParameter;
    setGuardMode(mode: GuardMode): Promise<void>;
    getCameraInfo(): Promise<void>;
    getStorageInfo(): Promise<void>;
    private onAlarmMode;
    private _getDeviceSerial;
    private onCameraInfo;
    private onCommandResponse;
    private onConnect;
    private onDisconnect;
    private onTimeout;
    private getCurrentDelay;
    private resetCurrentDelay;
    private scheduleReconnect;
    rebootHUB(): Promise<void>;
    setStatusLed(device: Device, value: boolean): Promise<void>;
    setAutoNightVision(device: Device, value: boolean): Promise<void>;
    setNightVision(device: Device, value: number): Promise<void>;
    setMotionDetection(device: Device, value: boolean): Promise<void>;
    setSoundDetection(device: Device, value: boolean): Promise<void>;
    setSoundDetectionType(device: Device, value: number): Promise<void>;
    setSoundDetectionSensitivity(device: Device, value: number): Promise<void>;
    setPetDetection(device: Device, value: boolean): Promise<void>;
    panAndTilt(device: Device, direction: PanTiltDirection, command?: number): Promise<void>;
    switchLight(device: Device, value: boolean): Promise<void>;
    setMotionDetectionSensitivity(device: Device, sensitivity: number): Promise<void>;
    setMotionDetectionType(device: Device, type: number): Promise<void>;
    setMotionTracking(device: Device, value: boolean): Promise<void>;
    setPanAndTiltRotationSpeed(device: Device, value: number): Promise<void>;
    setMicMute(device: Device, value: boolean): Promise<void>;
    setAudioRecording(device: Device, value: boolean): Promise<void>;
    enableSpeaker(device: Device, value: boolean): Promise<void>;
    setSpeakerVolume(device: Device, value: number): Promise<void>;
    setRingtoneVolume(device: Device, value: number): Promise<void>;
    enableIndoorChime(device: Device, value: boolean): Promise<void>;
    enableHomebaseChime(device: Device, value: boolean): Promise<void>;
    setHomebaseChimeRingtoneVolume(device: Device, value: number): Promise<void>;
    setHomebaseChimeRingtoneType(device: Device, value: number): Promise<void>;
    setNotificationType(device: Device, value: NotificationType): Promise<void>;
    setNotificationPerson(device: Device, value: boolean): Promise<void>;
    setNotificationPet(device: Device, value: boolean): Promise<void>;
    setNotificationAllOtherMotion(device: Device, value: boolean): Promise<void>;
    setNotificationAllSound(device: Device, value: boolean): Promise<void>;
    setNotificationCrying(device: Device, value: boolean): Promise<void>;
    setNotificationRing(device: Device, value: boolean): Promise<void>;
    setNotificationMotion(device: Device, value: boolean): Promise<void>;
    setPowerSource(device: Device, value: PowerSource): Promise<void>;
    setPowerWorkingMode(device: Device, value: number): Promise<void>;
    setRecordingClipLength(device: Device, value: number): Promise<void>;
    setRecordingRetriggerInterval(device: Device, value: number): Promise<void>;
    setRecordingEndClipMotionStops(device: Device, value: boolean): Promise<void>;
    setVideoStreamingQuality(device: Device, value: number): Promise<void>;
    setVideoRecordingQuality(device: Device, value: number): Promise<void>;
    setWDR(device: Device, value: boolean): Promise<void>;
    setFloodlightLightSettingsEnable(device: Device, value: boolean): Promise<void>;
    setFloodlightLightSettingsBrightnessManual(device: Device, value: number): Promise<void>;
    setFloodlightLightSettingsBrightnessMotion(device: Device, value: number): Promise<void>;
    setFloodlightLightSettingsBrightnessSchedule(device: Device, value: number): Promise<void>;
    setFloodlightLightSettingsMotionTriggered(device: Device, value: boolean): Promise<void>;
    setFloodlightLightSettingsMotionTriggeredDistance(device: Device, value: FloodlightMotionTriggeredDistance): Promise<void>;
    setFloodlightLightSettingsMotionTriggeredTimer(device: Device, seconds: number): Promise<void>;
    triggerStationAlarmSound(seconds: number): Promise<void>;
    resetStationAlarmSound(): Promise<void>;
    triggerDeviceAlarmSound(device: Device, seconds: number): Promise<void>;
    resetDeviceAlarmSound(device: Device): Promise<void>;
    setStationAlarmRingtoneVolume(value: number): Promise<void>;
    setStationAlarmTone(tone: AlarmTone): Promise<void>;
    setStationPromptVolume(value: number): Promise<void>;
    setStationNotificationSwitchMode(mode: NotificationSwitchMode, value: boolean): Promise<void>;
    setStationNotificationStartAlarmDelay(value: boolean): Promise<void>;
    setStationTimeFormat(value: TimeFormat): Promise<void>;
    setRTSPStream(device: Device, value: boolean): Promise<void>;
    setAntiTheftDetection(device: Device, value: boolean): Promise<void>;
    setWatermark(device: Device, value: WatermarkSetting1 | WatermarkSetting2 | WatermarkSetting3 | WatermarkSetting4): Promise<void>;
    enableDevice(device: Device, value: boolean): Promise<void>;
    startDownload(device: Device, path: string, cipher_id: number): Promise<void>;
    cancelDownload(device: Device): Promise<void>;
    startLivestream(device: Device, videoCodec?: VideoCodec): Promise<void>;
    stopLivestream(device: Device): Promise<void>;
    isLiveStreaming(device: Device): boolean;
    isDownloading(device: Device): boolean;
    quickResponse(device: Device, voice_id: number): Promise<void>;
    setChirpVolume(device: Device, value: number): Promise<void>;
    setChirpTone(device: Device, value: number): Promise<void>;
    setHDR(device: Device, value: boolean): Promise<void>;
    setDistortionCorrection(device: Device, value: boolean): Promise<void>;
    setRingRecord(device: Device, value: number): Promise<void>;
    lockDevice(device: Device, value: boolean): Promise<void>;
    setStationSwitchModeWithAccessCode(value: boolean): Promise<void>;
    setStationAutoEndAlarm(value: boolean): Promise<void>;
    setStationTurnOffAlarmWithButton(value: boolean): Promise<void>;
    startRTSPStream(device: Device): Promise<void>;
    stopRTSPStream(device: Device): Promise<void>;
    setMotionDetectionRange(device: Device, type: number): Promise<void>;
    setMotionDetectionRangeStandardSensitivity(device: Device, sensitivity: number): Promise<void>;
    setMotionDetectionRangeAdvancedLeftSensitivity(device: Device, sensitivity: number): Promise<void>;
    setMotionDetectionRangeAdvancedMiddleSensitivity(device: Device, sensitivity: number): Promise<void>;
    setMotionDetectionRangeAdvancedRightSensitivity(device: Device, sensitivity: number): Promise<void>;
    setMotionDetectionTestMode(device: Device, enabled: boolean): Promise<void>;
    setMotionTrackingSensitivity(device: Device, sensitivity: number): Promise<void>;
    setMotionAutoCruise(device: Device, enabled: boolean): Promise<void>;
    setMotionOutOfViewDetection(device: Device, enabled: boolean): Promise<void>;
    setLightSettingsColorTemperatureManual(device: Device, value: number): Promise<void>;
    setLightSettingsColorTemperatureMotion(device: Device, value: number): Promise<void>;
    setLightSettingsColorTemperatureSchedule(device: Device, value: number): Promise<void>;
    setLightSettingsMotionActivationMode(device: Device, value: number): Promise<void>;
    setVideoNightvisionImageAdjustment(device: Device, enabled: boolean): Promise<void>;
    setVideoColorNightvision(device: Device, enabled: boolean): Promise<void>;
    setAutoCalibration(device: Device, enabled: boolean): Promise<void>;
    isRTSPLiveStreaming(device: Device): boolean;
    setConnectionType(type: P2PConnectionType): void;
    getConnectionType(): P2PConnectionType;
    private onRuntimeState;
    private onChargingState;
    hasDevice(deviceSN: string): boolean;
    hasDeviceWithType(deviceType: DeviceType): boolean;
    private onFloodlightManualSwitch;
}
