/// <reference types="node" />
import { TypedEmitter } from "tiny-typed-emitter";
import NodeRSA from "node-rsa";
import { Address } from "./models";
import { CommandType, P2PDataType, P2PConnectionType } from "./types";
import { P2PClientProtocolEvents } from "./interfaces";
import { HubResponse } from "../http/models";
import { HTTPApi } from "../http/api";
export declare class P2PClientProtocol extends TypedEmitter<P2PClientProtocolEvents> {
    private readonly MAX_RETRIES;
    private readonly MAX_COMMAND_RESULT_WAIT;
    private readonly MAX_AKNOWLEDGE_TIMEOUT;
    private readonly MAX_LOOKUP_TIMEOUT;
    private readonly LOOKUP_RETRY_TIMEOUT;
    private readonly MAX_EXPECTED_SEQNO_WAIT;
    private readonly HEARTBEAT_INTERVAL;
    private readonly MAX_COMMAND_CONNECT_TIMEOUT;
    private readonly AUDIO_CODEC_ANALYZE_TIMEOUT;
    private readonly KEEPALIVE_INTERVAL;
    private readonly ESD_DISCONNECT_TIMEOUT;
    private readonly MAX_STREAM_DATA_WAIT;
    private readonly UDP_RECVBUFFERSIZE_BYTES;
    private readonly MAX_PAYLOAD_BYTES;
    private readonly MAX_PACKET_BYTES;
    private readonly MAX_VIDEO_PACKET_BYTES;
    private readonly P2P_DATA_HEADER_BYTES;
    private readonly MAX_SEQUENCE_NUMBER;
    private socket;
    private binded;
    private connected;
    private connecting;
    private terminating;
    private seqNumber;
    private lockSeqNumber;
    private expectedSeqNo;
    private currentMessageBuilder;
    private currentMessageState;
    private downloadTotalBytes;
    private downloadReceivedBytes;
    private cloudAddresses;
    private messageStates;
    private sendQueue;
    private connectTimeout?;
    private lookupTimeout?;
    private lookupRetryTimeout?;
    private heartbeatTimeout?;
    private keepaliveTimeout?;
    private esdDisconnectTimeout?;
    private connectTime;
    private lastPong;
    private connectionType;
    private fallbackAddresses;
    private energySavingDevice;
    private connectAddress;
    private dskKey;
    private dskExpiration;
    private log;
    private deviceSNs;
    private api;
    private rawStation;
    constructor(rawStation: HubResponse, api: HTTPApi);
    private _incrementSequence;
    private _initialize;
    private initializeMessageBuilder;
    private initializeMessageState;
    private _clearTimeout;
    private _clearMessageStateTimeouts;
    private _clearHeartbeatTimeout;
    private _clearKeepaliveTimeout;
    private _clearConnectTimeout;
    private _clearLookupTimeout;
    private _clearLookupRetryTimeout;
    private _clearESDDisconnectTimeout;
    private _disconnected;
    private closeEnergySavingDevice;
    private renewDSKKey;
    lookup(): void;
    lookup2(origAddress: Address, data: Buffer): void;
    private lookupByAddress;
    private lookupByAddress2;
    private lookupByAddress3;
    isConnected(): boolean;
    private _startConnectTimeout;
    private _connect;
    connect(): Promise<void>;
    private sendCamCheck;
    private sendCamCheck2;
    sendPing(address: Address): void;
    sendCommandWithIntString(commandType: CommandType, value: number, valueSub?: number, strValue?: string, strValueSub?: string, channel?: number): void;
    sendCommandWithInt(commandType: CommandType, value: number, strValue?: string, channel?: number): void;
    sendCommandWithStringPayload(commandType: CommandType, value: string, channel?: number): void;
    sendCommandWithString(commandType: CommandType, strValue: string, strValueSub: string, channel?: number): void;
    sendCommandPing(channel?: number): void;
    sendCommandDevicePing(channel?: number): void;
    private sendQueuedMessage;
    private sendCommand;
    private _sendCommand;
    private handleMsg;
    private parseDataMessage;
    private handleData;
    private isIFrame;
    private waitForStreamData;
    private handleDataBinaryAndVideo;
    private handleDataControl;
    private sendAck;
    private getDataType;
    close(): Promise<void>;
    private getHeartbeatInterval;
    private onClose;
    private onError;
    private scheduleHeartbeat;
    private scheduleP2PKeepalive;
    setDownloadRSAPrivateKeyPem(pem: string): void;
    getRSAPrivateKey(): NodeRSA | null;
    private initializeStream;
    private endStream;
    private endRTSPStream;
    private emitStreamStartEvent;
    private emitStreamStopEvent;
    isStreaming(channel: number, datatype: P2PDataType): boolean;
    isLiveStreaming(channel: number): boolean;
    isRTSPLiveStreaming(channel: number): boolean;
    isDownloading(channel: number): boolean;
    getLockSequenceNumber(): number;
    incLockSequenceNumber(): number;
    setConnectionType(type: P2PConnectionType): void;
    getConnectionType(): P2PConnectionType;
    isEnergySavingDevice(): boolean;
    private getDSKKeys;
    updateRawStation(value: HubResponse): void;
}
